import { Module } from '@nestjs/common';
import { ConfigModule } from '@nestjs/config';
import { TypeOrmModule } from '@nestjs/typeorm';
import { CONNECTION_NAME } from 'src/core/strings/constants/base.constants';

import { CqrsModule } from '@nestjs/cqrs';

import { ItemModel } from 'src/modules/item-related/item/data/models/item.model';
import {
  QueueItemModel,
  QueueOrderModel,
  QueueTicketModel,
} from './data/models/queue.model';
import { QueueController } from './infrastructure/controllers/queue.controller';
import { QueueOrchestrator } from './domain/queue.orchestrator';
import { QueueTransactionHandler } from './infrastructure/handlers/transaction.handler';
import { TransactionDataService } from '../transaction/transaction/data/services/transaction-data.service';
import { TransactionModel } from '../transaction/transaction/data/models/transaction.model';
import { TicketDataService } from './data/services/ticket.service';

@Module({
  imports: [
    ConfigModule.forRoot(),
    TypeOrmModule.forFeature(
      [
        QueueOrderModel,
        QueueTicketModel,
        QueueItemModel,
        ItemModel,
        TransactionModel,
      ],
      CONNECTION_NAME.DEFAULT,
    ),
    CqrsModule,
  ],
  controllers: [QueueController],
  providers: [
    QueueTransactionHandler,

    QueueOrchestrator,

    TransactionDataService,
    TicketDataService,
  ],
})
export class QueueModule {}
